% functions:
% radialcos2
function [x,mask1]=hilbert2d(xr,alpha,num)
%HILBERT2D  2D-Hilbert transform.
%           Computes 2-dimensional Hilbert transform of image XR.
%	    [X,MASK]=HILBERT2D(XR,ALPHA)
%           [X,MASK]=HILBERT2D(XR,ALPHA,NUM) 
%		   XR = Input real image.
%		ALPHA = Angle in degrees that defines "causal" plane in 
%			wavenumber domain.
%	            X = Complex image, where imag(X) is the Hilbert
%	     		transform of XR, and real(X) = XR.
%		 MASK = Mask used to select "causal" plane.
%                 NUM = Number of Orientations (default 2)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% file: hilbert2d.m
% author: Brian Sperry, Schepens Eye Research Inst.
% date: 20 Oct, 1994
% history: 
%    o 3/11/97 - added option to smooth the "causality" filter transition
%    o 5/08/98 - added number of orientations option (JN)
%
% notes:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 if (nargin < 3)
    num = 2;
 end

  SMOOTH = 1;
  alpha_rad = alpha/180*pi;

%********************************************************
% Compute 2D FFT.  Xr is conj-symmetric since x is real.
%
  [r,c]=size(xr);
  if (r~=c), error('hilbert2d - input matrix must be square');
  else N = r;
  end;
  Xr=fftshift(fft2(xr));
  N2 = N/2;

%****************************************************************************
% Treating Xr as the xfm of the real part of an image x=xr + j*xi, compute X
% Start by making Xr one-sided, as defined by alpha.  The cardinal cases,
% where alpha = 0,90,180,270, etc, are handled separately from arbitrary
% alpha's.  - WHY??
%         
%       alpha=0                      alpha=90
%                 1    N2       N             kx->
%            1   |*|*|*|*|y|*|*|*|      |*|*|*|*|y| | | |
%                |*|*|*|*|y|*|*|*|      |*|*|*|*|y| | | |
%                |*|*|*|*|y|*|*|*|      |*|*|*|*|y| | | |
%           N2   |*|*|*|*|y|*|*|*|   k  |*|*|*|*|y| | | |   0 = 2-dimensional DC
%         N2+1   | | | | |0|x|x|x|   y  |x|x|x|x|0| | | |   x = y-dir DC
%                | | | | | | | | |   |  |*|*|*|*| | | | |   y = x-dir DC
%                | | | | | | | | |      |*|*|*|*| | | | |
%            N   | | | | | | | | |      |*|*|*|*| | | | |
%
%       alpha=270                   
%                                   (2D FFT in standard form)
%                 1    N2       N             kx->
%            1   | | | | | |*|*|*|     |0|x|x|x|x|x|x|x|
%                | | | | | |*|*|*|     |y|*|*|*|*|*|*|*|
%                | | | | | |*|*|*|     |y|*|*|*|*|*|*|*|  
%           N2   | | | | | |*|*|*|   k |y|*|*|*|*|*|*|*|
%         N2+1   | | | | |0|x|x|x|   y |y|*|*|*|*|*|*|*|
%                | | | | |y|*|*|*|   | |y|*|*|*|*|*|*|*|
%                | | | | |y|*|*|*|     |y|*|*|*|*|*|*|*|
%            N   | | | | |y|*|*|*|     |y|*|*|*|*|*|*|*|
% 

if(alpha~= 0 & abs(alpha)~= 90 & abs(alpha)~= 180 & abs(alpha)~= 270), 
%% Presumably, N is even. Therefore define a new coordinate system based 
%% on the vertices, and therefore N+1 by N+1.
  x1=[1:N]; y1=[1:N]; 	% original system
  x2=[-N2:N2]; y2=[-N2:N2];

% Create mask in 2nd coord system. Move across image, comparing 
% y-index with y-value computed using alpha and x. 
  mask2 = zeros(N+1);

  for k=1:N+1,
   if k<=N2,
     mask2((y2<=round(-tan(alpha_rad)*x2(k))),k) = ...
		mask2((y2<=round(-tan(alpha_rad)*x2(k))),k) + 1;
   else,
     mask2((y2<=round(-tan(alpha_rad)*x2(k)+1)),k) = ...
		mask2((y2<=round(-tan(alpha_rad)*x2(k)+1)),k)+1;
   end;
  end;

% Now shift mask onto the original coordinate system. Corresponds to
% shifting mask2 towards negative y2 and positive x2 by 1/2 a step.
  mask1 = mask2(2:N+1,1:N);  

else,
%****************************************
% Special Cases - alpha = 0,90,180,270...
%
  mask1 = zeros(N);
  X = zeros(N);
  if (alpha == 0),
    % Keep upper-half plane...
    mask1(1:N2,1:N2)     = mask1(1:N2,1:N2)+1;
    mask1(1:N2+1,N2+1:N) = mask1(1:N2+1,N2+1:N)+1;
  elseif (alpha == 90)
    % Keep left-half plane...
    mask1(1:N,1:N2)     = mask1(1:N,1:N2)+1;
    mask1(N2+1:N,N2+1)  = mask1(N2+1:N,N2+1)+1;

  elseif (alpha == 180),
    % Keep lower-half plane...
    mask1(N2+2:N,1:N2+1) = mask1(N2+2:N,1:N2+1)+1;
    mask1(N2+1:N,N2+2:N) = mask1(N2+1:N,N2+2:N)+1;
  elseif (alpha == 270),
    % Keep right-half plane...
    mask1(1:N2,N2+2:N)   = mask1(1:N2,N2+2:N)+1;
    mask1(N2+1:N,N2+1:N) = mask1(N2+1:N,N2+1:N) +1;
  end;

end;

if(SMOOTH),
  F = radialcos2(N,N,alpha+90,180/num);
  F(N2+1,N2+1) = 1;
else,
  F = 1;
end;

  mask1 = 2*mask1; 
  mask1(N2+1,N2+1) = 1;  		% don't want dc point to have double energy!
  x = ifft2(fftshift( Xr .* F.* mask1 ));

  return;


