function [fimage,G] = logcosine(image,order,cscale,fscale)
% LOGCOSINE Bandpass filtering operation using log-cosine filter.
%                Y = LOGCOSINE(COMPLEXIMAGE,FILTER_ORDER)
%            [Y,G] = LOGCOSINE(COMPLEXIMAGE,FILTER_ORDER) 
%                where,  
%                Y = bandpass-filtered image (in spatial domain)
%                COMPLEXIMAGE = centered 2D-FFT of input image 
%
%				For randomized center/frequency:
%
%					  Y = LOGCOSINE(COMPLEXIMAGE,FILTER_ORDER,CSCALE)
%					  Y = LOGCOSINE(COMPLEXIMAGE,FILTER_ORDER,CSCALE,FSCALE)
%					  where,
%					  CSCALE = scale for randomized central frequency
%					  FSCALE = scale for randomized frequency
%						 in both cases, a value of zero implies no ranomization
%
%           See also: RADIALCOS2  

%  Written by J. Bondaryk
%  vectorized by BJS - much faster than original version!!!!
%
%  1/4/00 Jack Nye -- incorporated ability to randomize 
%                     center and frequency
%
%=============================================================================

[R,C]=size(image);
  if(R~=C), error('Not handling non-square images yet!'); end;

% Initialize
  G=zeros(R);
%
% Set up coordinate matrix in polar coordinates, centered at (C/2+1,R/2+1)
%
  x=1:C;
  X = x(ones(1,R),:); Y = X';
  r = sqrt((X-(C/2+1)).^2 + (Y-(R/2+1)).^2);
  r2 = r + .00001;	       % use r2 in testing log2(r) so don't get log(0) error.
  logr2 = log2(r2);
  
  % Randomized central freq: cf_new = cf_old * (1 + cscale * delta)
  %								  where delta is a random number -.5 to .5
  
  cf = order;
  
  if (nargin > 2 & cscale ~= 0)
     rand('state',sum(100*clock));
     cf = 2^order * (1 + cscale * (rand - .5));
     cf = log2(cf + .000001);
  end
    
  % Randomized frequency: freq = (1 + fscale * delta) 
  %                          where delta is a random number -.5 to .5
  
    if (nargin > 3 & fscale ~= 0)
     rand('state',sum(100*clock));%resets it to a different state each time (see help rand)
     freq = (1 + fscale * (rand - .5));
  else
     freq = 1;
  end
  
  
%
% Create filter
%
  if cf==0,
    ind = find( r~=0 & logr2<=1);
    G(ind) = 1 - 0.5*(1+cos(pi*log2(r(ind)) - pi ));
    ind = find( r==0 );
    G(ind) = G(ind) + 1;
  else
    ind = find( r ~= 0  &  logr2 < (cf*freq+1)/freq  &  logr2 > (cf*freq-1)/freq);
    G(ind) = 0.5*(1 + cos(freq * pi*log2(r(ind)) - freq * pi*cf));
  end
%
% Apply filter to image and transform back to space-domain. 
%
  fimage = real(ifft2(fftshift(image.*G)));

  return;
